# -*- coding: utf-8 -*-

import logging

import dbus

logger = logging.getLogger("main.network")

view_interface = "org.gnome.Sudoku.view"

class Peer(dbus.service.Object):

    def __init__(self, bus, obj_path, game):
        """game is the UI in main.py, for which we should get a better name
        """
        dbus.service.Object.__init__(self, bus, obj_path)
        self._game = game

    @dbus.service.method(dbus_interface=view_interface,
            in_signature="", out_signature="b")
    def has_won(self):
        return self._game.won

    @dbus.service.method(dbus_interface=view_interface,
            in_signature="", out_signature="")
    def acknowledge_win(self):
        """Used by the peer to tell us that we fail"""
        logger.info('our peer says we fail')
        self._game.fail_network_game()

class SideViewProxy(dbus.service.Object):
    """A dbus object, which exports a sudoku game's side-view on dbus
    """

    def __init__(self, bus, obj_path, mode, sudoku_view):
        dbus.service.Object.__init__(self, bus, obj_path)
        self._view = sudoku_view
        self._is_collaborate = mode

    @dbus.service.method(dbus_interface=view_interface,
            in_signature="iii", out_signature="")
    def set_value(self, x, y, value):
        if self._is_collaborate:
            self._view.set_value(x, y, value)
        # else is competitive mode
        elif value:
            self._view.set_invisible(x, y)
        else:
            self._view.set_empty(x, y)

    @dbus.service.method(dbus_interface=view_interface,
            in_signature="ii", out_signature="")
    def set_focus(self, x, y):
        self._view.set_focus(x, y)

class RemoteViewHandler:

    """A handler which has identical interface with SudokuView and will relay
    commands to a remote SideViewProxy
    """

    def __init__(self, remote_proxy):
        """remote_proxy is a SideViewProxy"""
        self._remote_proxy = remote_proxy

    def update(self, changes):
        def reply_handler():
            pass
        def error_handler(error):
            logger.error('Failed to update the remote view: %s' % error)

        for box in changes:
            x, y, value = box.x, box.y, box.value
            if value is None:
                pass
            else:
                self._remote_proxy.set_value(x, y, value,
                        reply_handler=reply_handler,
                        error_handler=error_handler)

    def set_focus(self, x, y):
        def reply_handler():
            pass
        def error_handler(error):
            logger.error('Failed to set_focus: %s' % error)

        self._remote_proxy.set_focus(x, y,
                reply_handler=reply_handler,
                error_handler=error_handler)
