# -*- coding: utf-8 -*-

class HistoryManager:
    def __init__(self, undo_action, redo_action):
        self._undo_action = undo_action
        self._redo_action = redo_action
        self.clear()

    def execute_command(self, cmd):
        # first clear any undone items from the history list
        self._trim_history_list()
        cmd.execute()
        self._history.append(cmd)
        self._next_undo += 1
        self._update_gui()

    def undo(self):
        # nothing to undo
        if self._next_undo < 0:
            return

        command = self._history[self._next_undo]
        command.undo()
        self._next_undo -= 1
        self._update_gui()

    def redo(self):
        # nothing to redo
        if self._next_undo == len(self._history) - 1:
            return

        cmd = self._history[self._next_undo + 1]
        cmd.execute()

        self._next_undo += 1
        self._update_gui()

    def clear(self):
        self._history = []
        self._next_undo = -1
        self._update_gui()

    def _trim_history_list(self):
        """Purge all undone commands from the history list
        """
        del self._history[(self._next_undo + 1):]
        self._update_gui()

    def _update_gui(self):
        undo, redo = True, True
        if not self._history:
            undo, redo = False, False
        elif self._next_undo < 0:
            undo = False
        if self._next_undo == len(self._history) - 1:
            redo = False

        self._undo_action.set_sensitive(undo)
        self._redo_action.set_sensitive(redo)
